@@ -1,11 +1,13 @@
-"""Module containing the Particles class for creating and storing
+"""
+Module containing the Particles class for creating and storing
 and initial particle distribution.
 
 Module also contains utility functions for particle distribution
 like assertions, conversions and filtering of lost particles.
 
-:Author(s): **Christoffer Hjertø Grindheim**"""
-import logging as log
+:Author(s): **Christoffer Hjertø Grindheim**, **Anton Lu**
+"""
+import logging
 from typing import Tuple, Sequence, TYPE_CHECKING, Union
 
 import numpy as np
@@ -15,6 +17,9 @@ from .. import assertions as asrt, exceptions as expt
 
 if TYPE_CHECKING:
     from .machine import Machine
+    from .machine_base import MachineABC
+
+log = logging.getLogger(__name__)
 
 
 class Particles(object):
@@ -87,8 +92,8 @@ class Particles(object):
                                  coordinates: Tuple[np.ndarray, np.ndarray]):
         """
         See the definition of the property
-        :py:func`longitudinal_tomography.tracking.particles.coordinates_dphi_denergy` for full
-        documentation.
+        :py:func`longitudinal_tomography.tracking.particles.
+                 coordinates_dphi_denergy` for full documentation.
 
         Parameters
         ----------
@@ -104,12 +109,14 @@ class Particles(object):
         """
         self._dphi, self._denergy = _assert_coordinates(coordinates)
 
-    def homogeneous_distribution(self, machine: 'Machine', recprof: int):
+    def homogeneous_distribution(self, machine: 'Machine', recprof: int,
+                                 deltaturn: int = 0):
         """Function for automatic generation of particle distribution.
 
         The distributions created are identical to the distributions created
         in the Fortran tomography. The reconstruction area is found by calling
-        :func:`~longitudinal_tomography.tracking.phase_space_info.PhaseSpaceInfo.find_binned_phase_energy_limits`.
+        :func:`~longitudinal_tomography.tracking.phase_space_info.
+                PhaseSpaceInfo.find_binned_phase_energy_limits`.
 
         Parameters
         ----------
@@ -120,6 +127,9 @@ class Particles(object):
         recprof: int
             The index of the profile (time frame) to be reconstructed.
             This will be the profile where the distribution is generated.
+        deltaturn: int
+            On offset in turn numbers to allow creation of distributions
+            between two recorded profiles
 
         Raises
         ------
@@ -170,7 +180,8 @@ class Particles(object):
         self.jmax = psinfo.jmax
 
         # Converting from phase space coordinates to physical units.
-        coords = self._bin_nr_to_physical_coords(coords, machine, recprof)
+        coords = self._bin_nr_to_physical_coords(coords, machine, recprof,
+                                                 deltaturn)
         self.coordinates_dphi_denergy = coords
 
     def _bin_nr_to_physical_coords(self,
@@ -178,7 +189,8 @@ class Particles(object):
                                        Sequence[np.ndarray],
                                        np.ndarray
                                    ],
-                                   machine: 'Machine', recprof: int) \
+                                   machine: 'Machine', recprof: int,
+                                   deltaturn: int = 0) \
             -> Tuple[np.ndarray, np.ndarray]:
         """Function to convert from reconstructed phase space coordinates
         to physical units.
@@ -195,8 +207,11 @@ class Particles(object):
             Machine object holding machine parameters and settings
         recprof: int
             Profile to reconstruct.
+        deltaturn: int
+            On offset in turn numbers to allow creation of distributions
+            between two recorded profiles
         """
-        turn = recprof * machine.dturns
+        turn = recprof * machine.dturns + deltaturn
         dphi = ((coordinates[0] + self.xorigin)
                 * machine.h_num * machine.omega_rev0[turn] * machine.dtbin
                 - machine.phi0[turn])
@@ -209,8 +224,7 @@ class Particles(object):
         needed_fieds = ['snpt', 'h_num', 'omega_rev0', 'eta0',
                         'dtbin', 'phi0', 'synch_part_y', 'dturns', 'phi12',
                         'nbins', 'beam_ref_frame', 'full_pp_flag',
-                        'demax', 'vrf2', 'vrf2dot', 'e0', 'vrf1',
-                        'vrf1dot', 'min_dt', 'max_dt', 'time_at_turn']
+                        'demax', 'e0', 'min_dt', 'max_dt', 'time_at_turn']
         asrt.assert_fields(
             machine, 'machine', needed_fieds, expt.MachineParameterError,
             'Did you remember to use machine.values_at_turns()?')
@@ -269,7 +283,7 @@ def filter_lost(xp: np.ndarray, yp: np.ndarray, img_width: int) \
 
 
 def physical_to_coords(tracked_dphi: np.ndarray, tracked_denergy: np.ndarray,
-                       machine: 'Machine', xorigin: float, dEbin: float) \
+                       machine: 'MachineABC', xorigin: float, dEbin: float) \
         -> Tuple[np.ndarray, np.ndarray]:
     """Function to convert from physical units ([rad], [eV]) to reconstructed
     phase space coordinates (bin numbers).
@@ -287,7 +301,7 @@ def physical_to_coords(tracked_dphi: np.ndarray, tracked_denergy: np.ndarray,
         2D array containing the energy difference [eV] relative to
         the synchronous particle for every particle at every time frame.
         Array shape: (nprofiles, nparticles)
-    machine: Machine
+    machine: MachineABC
         machine object holding machine parameters, and settings for
         the reconstruction.
     xorigin: float
