@@ -1,18 +1,20 @@
 """Module containing the Tracking class.
 
-:Author(s): **Christoffer Hjertø Grindheim**
+:Author(s): **Christoffer Hjertø Grindheim**, **Anton Lu**
 """
-from typing import Tuple, TYPE_CHECKING
+from typing import Tuple, TYPE_CHECKING, Callable
 
 import numpy as np
 import logging
+import warnings
 
 from .. import assertions as asrt
 from .__tracking import ParticleTracker
-from ..cpp_routines import tomolib_wrappers as tlw
+from ..cpp_routines import libtomo
 from ..compat import fortran
 
 if TYPE_CHECKING:
+    from .machine_base import MachineABC
     from .machine import Machine
 
 log = logging.getLogger(__name__)
@@ -49,11 +51,11 @@ class Tracking(ParticleTracker):
         stdout during particle tracking.
     """
 
-    def __init__(self, machine: 'Machine'):
+    def __init__(self, machine: 'MachineABC'):
         super().__init__(machine)
 
-    def track(self, recprof: int,
-              init_distr: Tuple[float, float] = None) \
+    def track(self, recprof: int, init_distr: Tuple[float, float] = None,
+              callback: Callable = None, deltaturn: int = 0) \
             -> Tuple[np.ndarray, np.ndarray]:
         """Primary function for tracking particles.
 
@@ -64,7 +66,9 @@ class Tracking(ParticleTracker):
 
         By default, an distribution of particles is spread out homogeneously
         over the area to be reconstructed. This area is found using the
-        :class:`longitudinal_tomography.tracking.phase_space_info.PhaseSpaceInfo` class.
+        :class:
+            `longitudinal_tomography.tracking.phase_space_info.PhaseSpaceInfo`
+            class.
         The homogeneous distribution is placed on the time frame intended
         to be reconstructed for optimum quality. This is based on the
         original tomography algorithm.
@@ -73,13 +77,17 @@ class Tracking(ParticleTracker):
         default, automatic generation of particles.
 
         By calling
-        :func:`longitudinal_tomography.tracking.__tracking.ParticleTracker.enable_self_fields`,
+        :func:
+            `longitudinal_tomography.tracking.__tracking.ParticleTracker.
+            enable_self_fields`,
         a flag indicating that self-fields should be included is set.
         In this case, :func:`kick_and_drift_self` will be used.
         Note that tracking including self fields are much slower than without.
 
         By calling
-        :func:`longitudinal_tomography.tracking.__tracking.ParticleTracker.enable_fortran_output`,
+        :func:
+            `longitudinal_tomography.tracking.__tracking.ParticleTracker.
+            enable_fortran_output`,
         an output resembling the original is written to stdout.
         Note that the values for the number of lost particles is **not valid**.
         Note also, that if the full Fortran output is to be printed,
@@ -96,6 +104,11 @@ class Tracking(ParticleTracker):
             coordinates (dphi, denergy). dphi is the phase difference
             from the synchronous particle [rad]. denergy is the difference
             in energy from the synchronous particle [eV].
+        callback: Callable
+            Passing a callback with function signature
+            (progress: int, total: int) will allow the tracking loop to call
+            this function at the end of each turn, allowing the python caller
+            to monitor the progress.
 
         Returns
         -------
@@ -118,20 +131,44 @@ class Tracking(ParticleTracker):
               given as energy [eV] relative to the synchronous particle.
         """
 
+        if deltaturn != 0:
+            warnings.warn("deltaturn is still an experimental "
+                          + "feature, results may not be reliable and it may"
+                          + " not be maintained.")
+
         recprof = asrt.assert_index_ok(
             recprof, self.machine.nprofiles, wrap_around=True)
+        machine = self.machine
+
+        asrt.assert_less(np.abs(deltaturn), '|deltaturn|', machine.dturns,
+                         ValueError, '|deltaturn| must be less than the'
+                         + ' number of turns between profiles')
+
+        asrt.assert_less(machine.dturns*recprof + deltaturn,
+                         'reconstruction turn',
+                         machine.dturns*(machine.nprofiles-1), ValueError,
+                         'Reconstruction turn '
+                         + '(machine.dturns*recprof + deltaturn) must be less'
+                         + ' than the total number of measured turns.')
+
+        asrt.assert_greater_or_equal(machine.dturns*recprof + deltaturn,
+                                     'reconstruction turn', 0, ValueError,
+                                     'Reconstruction turn '
+                                     + '(machine.dturn*recprof + deltaturn) '
+                                     + 'must be greater than 0.')
 
         if init_distr is None:
             # Homogeneous distribution is created based on the
             # original Fortran algorithm.
             log.info('Creating homogeneous distribution of particles.')
-            self.particles.homogeneous_distribution(self.machine, recprof)
+            self.particles.homogeneous_distribution(machine, recprof,
+                                                    deltaturn)
             coords = self.particles.coordinates_dphi_denergy
 
             # Print fortran style plot info. Needed for tomograph.
             if self.fortran_flag:
                 print(fortran.write_plotinfo(
-                    self.machine, self.particles, self._profile_charge))
+                    machine, self.particles, self._profile_charge))
 
         else:
             log.info('Using initial particle coordinates set by user.')
@@ -141,8 +178,11 @@ class Tracking(ParticleTracker):
         dphi = coords[0]
         denergy = coords[1]
 
-        rfv1 = self.machine.vrf1_at_turn * self.machine.q
-        rfv2 = self.machine.vrf2_at_turn * self.machine.q
+        self.init_dphi = dphi.copy()
+        self.init_denergy = denergy.copy()
+
+        rfv1 = machine.vrf1_at_turn * machine.q
+        rfv2 = machine.vrf2_at_turn * machine.q
 
         # Tracking particles
         if self.self_field_flag:
@@ -155,22 +195,24 @@ class Tracking(ParticleTracker):
         else:
             # Tracking without self-fields
             nparts = len(dphi)
-            nturns = self.machine.dturns * (self.machine.nprofiles - 1)
-            xp = np.zeros((self.machine.nprofiles, nparts))
-            yp = np.zeros((self.machine.nprofiles, nparts))
+            nturns = machine.dturns * (machine.nprofiles - 1)
+            xp = np.zeros((machine.nprofiles, nparts))
+            yp = np.zeros((machine.nprofiles, nparts))
 
             # Calling C++ implementation of tracking routine.
-            xp, yp = tlw.kick_and_drift(
-                xp, yp, denergy, dphi, rfv1, rfv2, recprof,
-                nturns, nparts, machine=self.machine,
-                ftn_out=self.fortran_flag)
+            libtomo.kick_and_drift(xp, yp, denergy, dphi, rfv1, rfv2,
+                                   machine.phi0, machine.deltaE0,
+                                   machine.drift_coef, machine.phi12,
+                                   machine.h_ratio, machine.dturns,
+                                   recprof, deltaturn, nturns, nparts,
+                                   self.fortran_flag, callback=callback)
 
         log.info('Tracking completed!')
         return xp, yp
 
     def kick_and_drift(self, denergy: np.ndarray, dphi: np.ndarray,
-                       rf1v: np.ndarray, rf2v: np.ndarray, rec_prof: int) -> \
-            Tuple[np.ndarray, np.ndarray]:
+                       rf1v: np.ndarray, rf2v: np.ndarray, rec_prof: int,
+                       deltaturn: int = 0) -> Tuple[np.ndarray, np.ndarray]:
         """Routine for tracking a distribution of particles for N turns.
         N is given by *tracking.nturns*
 
@@ -191,6 +233,9 @@ class Tracking(ParticleTracker):
             multiplied by particle charge
         rec_prof: int
             Time slice to initiate tracking.
+        deltaturn: int
+            On offset in turn numbers in case the initial distribution is
+            between profiles.
 
         Returns
         -------
@@ -208,22 +253,30 @@ class Tracking(ParticleTracker):
         out_denergy = np.copy(out_dphi)
 
         # Setting homogeneous coordinates to profile to be reconstructed.
-        out_dphi[rec_prof] = np.copy(dphi)
-        out_denergy[rec_prof] = np.copy(denergy)
+        init_dphi = np.copy(dphi)
+        init_denergy = np.copy(denergy)
 
-        rec_turn = rec_prof * self.machine.dturns
+        if deltaturn == 0:
+            out_dphi[rec_prof] = np.copy(dphi)
+            out_denergy[rec_prof] = np.copy(denergy)
+
+        rec_turn = rec_prof * self.machine.dturns + deltaturn
         turn = rec_turn
+
         profile = rec_prof
 
+        if deltaturn < 0:
+            profile -= 1
+
         # Tracking 'upwards'
         while turn < self.nturns:
             # Calculating change in phase for each particle at a turn
-            dphi = tlw.drift(denergy, dphi, self.machine.drift_coef,
-                             nparts, turn)
+            dphi = libtomo.drift(denergy, dphi, self.machine.drift_coef,
+                                 nparts, turn)
             turn += 1
             # Calculating change in energy for each particle at a turn
-            denergy = tlw.kick(self.machine, denergy, dphi, rf1v, rf2v,
-                               nparts, turn)
+            denergy = libtomo.kick(self.machine, denergy, dphi, rf1v, rf2v,
+                                   nparts, turn)
 
             if turn % self.machine.dturns == 0:
                 profile += 1
@@ -233,20 +286,20 @@ class Tracking(ParticleTracker):
                     fortran.print_tracking_status(rec_prof, profile)
 
         # Starting again from homogenous distribution
-        dphi = np.copy(out_dphi[rec_prof])
-        denergy = np.copy(out_denergy[rec_prof])
+        dphi = np.copy(init_dphi)
+        denergy = np.copy(init_denergy)
         turn = rec_turn
         profile = rec_prof
 
         # Tracking 'downwards'
         while turn > 0:
             # Calculating change in energy for each particle at a turn
-            denergy = tlw.kick(self.machine, denergy, dphi, rf1v, rf2v,
-                               nparts, turn, up=False)
+            denergy = libtomo.kick(self.machine, denergy, dphi, rf1v, rf2v,
+                                   nparts, turn, up=False)
             turn -= 1
             # Calculating change in phase for each particle at a turn
-            dphi = tlw.drift(denergy, dphi, self.machine.drift_coef,
-                             nparts, turn, up=False)
+            dphi = libtomo.drift(denergy, dphi, self.machine.drift_coef,
+                                 nparts, turn, up=False)
 
             if turn % self.machine.dturns == 0:
                 profile -= 1
@@ -259,14 +312,16 @@ class Tracking(ParticleTracker):
 
     def kick_and_drift_self(self, denergy: np.ndarray, dphi: np.ndarray,
                             rf1v: np.ndarray, rf2v: np.ndarray,
-                            rec_prof: int) -> Tuple[np.ndarray, np.ndarray]:
+                            rec_prof: int, deltaturn: int = 0) \
+            -> Tuple[np.ndarray, np.ndarray]:
         """Routine for tracking a given distribution of particles,\
         including self-fields. Implemented as hybrid between Python and C++.
 
         Routine for tracking, with self-fields, a distribution of
         particles for N turns. N is given by *tracking.nturns*.
 
-        Used by the function :py:meth:`longitudinal_tomography.tracking.tracking.Tracking.track`
+        Used by the function
+        :py:meth:`longitudinal_tomography.tracking.tracking.Tracking.track`
         to track using self-fields.
 
         Returns the coordinates of the particles as phase space coordinates
@@ -289,6 +344,9 @@ class Tracking(ParticleTracker):
             multiplied by particle charge
         rec_prof: int
             Time slice to initiate tracking.
+        deltaturn: int
+            On offset in turn numbers in case the initial distribution is
+            between profiles.
 
         Returns
         -------
@@ -307,10 +365,13 @@ class Tracking(ParticleTracker):
         xp = np.zeros((self.machine.nprofiles, nparts))
         yp = np.copy(xp)
 
-        rec_turn = rec_prof * self.machine.dturns
+        rec_turn = rec_prof * self.machine.dturns + deltaturn
         turn = rec_turn
         profile = rec_prof
 
+        if deltaturn < 0:
+            profile -= 1
+
         # To be saved for downwards tracking
         dphi0 = np.copy(dphi)
         denergy0 = np.copy(denergy)
@@ -327,8 +388,8 @@ class Tracking(ParticleTracker):
         if self.fortran_flag:
             fortran.print_tracking_status(rec_prof, profile)
         while turn < self.nturns:
-            dphi = tlw.drift(denergy, dphi, self.machine.drift_coef,
-                             nparts, turn)
+            dphi = libtomo.drift(denergy, dphi, self.machine.drift_coef,
+                                 nparts, turn)
 
             turn += 1
 
@@ -340,8 +401,8 @@ class Tracking(ParticleTracker):
                                        self._phiwrap)
             selfvolt = self._vself[profile, temp_xp.astype(int) - 1]
 
-            denergy = tlw.kick(self.machine, denergy, dphi, rf1v, rf2v,
-                               nparts, turn)
+            denergy = libtomo.kick(self.machine, denergy, dphi, rf1v, rf2v,
+                                   nparts, turn)
             denergy += selfvolt * self.machine.q
 
             if turn % self.machine.dturns == 0:
@@ -356,20 +417,21 @@ class Tracking(ParticleTracker):
         denergy = denergy0
         turn = rec_turn
         profile = rec_prof - 1
+
         temp_xp = xp[rec_prof]
 
         while turn > 0:
             selfvolt = self._vself[profile, temp_xp.astype(int)]
 
-            denergy = tlw.kick(self.machine, denergy, dphi, rf1v, rf2v,
-                               nparts, turn, up=False)
+            denergy = libtomo.kick(self.machine, denergy, dphi, rf1v, rf2v,
+                                   nparts, turn, up=False)
 
             denergy += selfvolt * self.machine.q
 
             turn -= 1
 
-            dphi = tlw.drift(denergy, dphi, self.machine.drift_coef,
-                             nparts, turn, up=False)
+            dphi = libtomo.drift(denergy, dphi, self.machine.drift_coef,
+                                 nparts, turn, up=False)
 
             temp_xp = self._calc_xp_sf(
                 dphi, self.machine.phi0[turn], self.particles.xorigin,
